#include "esp_system.h"
#include <WiFi.h>
#include <Wire.h>
#include <WiFiUdp.h>
#include <Adafruit_PWMServoDriver.h>
#include "SSD1306.h"//ディスプレイ用ライブラリを読み込み
#define MAXSN 4       // Max Number of Servos
#define MAXFN 5       // Max Number of Frames
#define MAXMN 5       // Max Number of Motions
#define SERVOMIN 220  // this is the 'minimum' pulse length count (out of 4096)
#define SERVOMAX 420  // this is the 'maximum' pulse length count (out of 4096)
#define FREQ 60       // PWM frequency
#define RESO 6        // motion resolution
#define R_PORT 9000
#define GPIN1 2
#define RPIN1 4
#define GPIN2 5
#define RPIN2 19

SSD1306  display(0x3c, 21, 22); //SSD1306インスタンスの作成（I2Cアドレス,SDA,SCL）

String lip;
String msg="";

WiFiServer server(80);
WiFiUDP udp;
// called this way, it uses the default address 0x40
Adafruit_PWMServoDriver pwm = Adafruit_PWMServoDriver();

// グローバル変数を定義します。
// 各サーボモータの補正値のテーブル
int trim[MAXSN] = { 0, // R Right
                    0, // L Left
                    0, // Head
                    0  // Tail
                   }; 
// 各モーションのサーボモータの動きのテーブル。
uint8_t motion[MAXMN][MAXFN][MAXSN]={
{ // 0 Stop
  { 90, 90, 90, 90},
  { 90, 90, 90, 90},
  { 90, 90, 90, 90},
  { 90, 90, 90, 90},
  { 90, 90, 90, 90}
},
{ // 1 Forward
  { 70, 90, 30, 30},
  { 90, 110, 90, 90},
  { 70, 90 ,150, 150},
  { 90, 110, 90, 90},
  { 90, 90, 90, 90}
},
{ // 2 Back
  { 90, 70, 150 ,150 },
  { 110, 90, 90 ,90},
  { 90, 70, 30 ,30},
  { 110, 90, 90, 90},
  { 90, 90, 90, 90}
},
{ // 3 Right
  { 90, 110, 30 ,30},
  { 110, 90, 90 ,90},
  { 90, 110, 150 ,150},
  { 110, 90, 90 ,90},
  { 90, 90, 90, 90}
},
{ // 4 Left
  { 70, 90, 150 ,150},
  { 90, 70, 90 ,90},
  { 70, 90, 30 ,30},
  { 90, 70, 90 ,90},
  { 90, 90, 90, 90}
}
};

// サーボモータの現在の角度
int nowAngle[MAXSN] = { 0, 0, 0, 0 }; // Initialize array to 0
// サーボモータが到達する角度
int targetAngle[MAXSN] = { 0, 0, 0,0 }; // Initialize array to 0
// サーボモータが単位時間（motion配列に定義されている時間の1/10）あたりに動作する角度
int deltaAngle[MAXSN] = { 0, 0, 0, 0 }; // Initialize array to 0
// 動作中のframeの番号を保持
uint8_t frameNumber = 0;
// 動作中のmotionの番号を保持
uint8_t motionNumber = 0;
// シリアルモニタから送られてきたコマンドを保持
uint8_t cmd = 0;

// シリアルモニタから送られてくる文字列を格納する変数
String inputString = "";         // a string to hold incoming data
// シリアルモニタからデータを受信したことを示す変数
boolean stringComplete = false;  // whether the string is complete

// サーボモータを回転させる関数です。
void servoWrite( int servo, int deg ) {
// 回転させる角度に補正値を加算してから、PWM信号を変化させるときの値に変換
  deg = map( ( deg + trim[servo] ), 0, 180, SERVOMIN, SERVOMAX );
// サーボモータを回転
  pwm.setPWM( servo, 0, deg );
}
int wadr=0;

void setup() {
  const char* ssid;
  const char* password;
  
  Serial.println();
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);
  int count=100;
  int success=1;
  ssid = "**********";
  password = "**********";
  WiFi.begin(ssid, password);
  WiFi.mode(WIFI_STA);                 // 無線LANをSTAモードに設定
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
    count--;
    if (count<0) {
      success=0;
      break;
    }
  }
  if (success==0){
    ssid = "**********";
    password = "**********";
    WiFi.begin(ssid, password);
    WiFi.mode(WIFI_STA);                 // 無線LANをSTAモードに設定
    while (WiFi.status() != WL_CONNECTED) {
      delay(500);
      Serial.print(".");
    }
  }
  Serial.println("");
  Serial.println("WiFi connected");
  Serial.println("IP address: ");
  lip=WiFi.localIP().toString();
  Serial.println(lip);
  display.setFont(ArialMT_Plain_16);    //フォントを設定
  server.begin();
  drawLcd(1);
  delay(200);
  pinMode(GPIN1,OUTPUT);
  pinMode(RPIN1,OUTPUT);
  pinMode(GPIN2,OUTPUT);
  pinMode(RPIN2,OUTPUT);
  server.begin();
  // サーボシールドを初期化。
  pwm.begin();

  // PWM信号の周波数を設定
  pwm.setPWMFreq( FREQ ); // Analog servos run at ~60 Hz updates

  // 初期化のためのループ
  for( int i = 0; i < MAXSN; i++) {
    // 各サーボモータを初期位置（90°）に回転。
    servoWrite( i, 90 );
    // targetAngle変数を初期化。
    targetAngle[i] = 90 * RESO;
  }
  udp.begin(R_PORT);
  drawLcd(0);  
}
 
void loop() {
  char lcdp[20];                          // 表示用変数を定義(49バイト48文字)
  int len;                                // 文字列長を示す整数型変数を定義
  memset(lcdp, 0, 20);                    // 文字列変数lcdの初期化(49バイト)
  len = udp.parsePacket();                // 受信パケット長を変数lenに代入
  if(len>0){                       // 未受信のときはloop()の先頭に戻る
    udp.read(lcdp, 20);
    motionNumber=0;
    // 受信データを文字列変数lcdpへ代入
    msg=lcdp;
    if (msg=="S") {
      motionNumber=0;
      digitalWrite( GPIN1,LOW);
      digitalWrite( RPIN1,LOW);
      digitalWrite( GPIN2,LOW);
      digitalWrite( RPIN2,LOW);
    }
    if (msg=="F") {
      motionNumber=1;
      digitalWrite( GPIN1,HIGH);
      digitalWrite( RPIN1,LOW);
      digitalWrite( GPIN2,HIGH);
      digitalWrite( RPIN2,LOW);
    }
    if (msg=="B") {
      motionNumber=2;
      digitalWrite( GPIN1,LOW);
      digitalWrite( RPIN1,HIGH);
      digitalWrite( GPIN2,LOW);
      digitalWrite( RPIN2,HIGH);
    }
    if (msg=="R") {
      motionNumber=3;
      digitalWrite( GPIN1,LOW);
      digitalWrite( RPIN1,HIGH);
      digitalWrite( GPIN2,HIGH);
      digitalWrite( RPIN2,LOW);
 
    }
    if (msg=="L") {
      motionNumber=4;
      digitalWrite( GPIN1,HIGH);
      digitalWrite( RPIN1,LOW);
      digitalWrite( GPIN2,LOW);
      digitalWrite( RPIN2,HIGH);

    }
    drawLcd(motionNumber);
    // フレームを実行するループ
    for ( frameNumber = 0; frameNumber < MAXFN; frameNumber++) {
      // 各変数を代入するループ
      for( int i = 0; i < MAXSN; i++ ) {
        // nowAngle変数を初期化
        nowAngle[i] = targetAngle[i];
        // targetAngle変数に次に到達する角度値を代入
        targetAngle[i] = (int)motion[motionNumber][frameNumber][i] * RESO;
        // deltaAngle変数の値を算出
        deltaAngle[i] = ( targetAngle[i] - nowAngle[i] ) / RESO;
      }

      // サーボモータを回転するループ
      for ( int j = 0; j < RESO; j++ ) {
        for( int i = 0; i < MAXSN; i++ ) {
          // nowAngle変数を更新
          nowAngle[i] += deltaAngle[i];
          // サーボモータを回転
          servoWrite( i, ( nowAngle[i] / RESO ) );
        }
        // 20msecウェイト
        delay(20);
      }
      delay(300);
    }
    drawLcd(0);
    digitalWrite( GPIN1,LOW);
    digitalWrite( RPIN1,LOW);
    digitalWrite( GPIN2,LOW);
    digitalWrite( RPIN2,LOW);
  }
}

void drawLcd(int s){
  display.init();    //ディスプレイを初期化
  display.drawString(0, 0, "TURTLE");
  display.drawString(0, 16, lip);  
  switch (s){
    case 0: display.drawString(0, 40,"Stop"); break;
    case 1: display.drawString(0, 40,"Front"); break;
    case 2: display.drawString(0, 40,"Back"); break;
    case 3: display.drawString(0, 40,"Right"); break;
    case 4: display.drawString(0, 40,"Left"); break;
  }
  display.display();   //指定された情報を描画
}
